@@ -1,5 +1,6 @@
 /* This file is part of KNemo
    Copyright (C) 2004, 2005, 2006 Percy Leonhardt <percy@eris23.de>
+   Copyright (C) 2009 John Stamp <jstamp@users.sourceforge.net>
 
    KNemo is free software; you can redistribute it and/or modify
    it under the terms of the GNU Library General Public License as
@@ -20,18 +21,15 @@
 #ifndef CONFIGDIALOG_H
 #define CONFIGDIALOG_H
 
-#include <qdict.h>
-#include <qpair.h>
-#include <qstring.h>
-#include <qlistview.h>
-
 #include <kcmodule.h>
 
 #include "data.h"
+#include "ui_configdlg.h"
 
+class QStandardItem;
+class QStandardItemModel;
 class QTimer;
-class ConfigDlg;
-class KNemoCheckListItem;
+class QTreeWidgetItem;
 
 /**
  * This is the configuration dialog for KNemo
@@ -50,7 +48,7 @@ public:
     /**
      * Default Constructor
      */
-    ConfigDialog( QWidget *parent, const char *name, const QStringList& );
+    ConfigDialog( QWidget *parent, const QVariantList &args );
 
     /**
      * Default Destructor
@@ -71,8 +69,7 @@ private slots:
     void buttonAddToolTipSelected();
     void buttonRemoveToolTipSelected();
     void buttonNotificationsSelected();
-    void buttonStatisticsDirSelected();
-    void interfaceSelected( const QString& interface );
+    void interfaceSelected( int row );
     void aliasChanged( const QString& text );
     void iconSetChanged( int set );
     void backendChanged( int set );
@@ -82,9 +79,8 @@ private slots:
     void checkBoxStartKNemoToggled( bool on );
     void spinBoxTrafficValueChanged( int value );
     void checkBoxCustomToggled( bool on );
-    void listViewCommandsSelectionChanged();
-    void listViewCommandsCheckListItemChanged( KNemoCheckListItem* item, bool state );
-    void listViewCommandsRenamed( QListViewItem* item, int col, const QString & text );
+    void listViewCommandsSelectionChanged( QTreeWidgetItem *current, QTreeWidgetItem *previous );
+    void listViewCommandsChanged( QTreeWidgetItem* item, int column );
 
     /**
      * These three are generic.
@@ -110,14 +106,16 @@ private:
 
     int mToolTipContent;
     bool mLock;
-    ConfigDlg* mDlg;
+    Ui::ConfigDlg* mDlg;
     QColor mColorVLines;
     QColor mColorHLines;
     QColor mColorIncoming;
     QColor mColorOutgoing;
     QColor mColorBackground;
-    QDict<InterfaceSettings> mSettingsDict;
-    QPair<QString, int> mToolTips[25];
+    KSharedConfigPtr mConfig;
+    QMap<QString, InterfaceSettings *> mSettingsDict;
+    QMap<quint32, QString> mToolTips;
+    QList<QString> mDeletedIfaces;
 
     static const QString ICON_DISCONNECTED;
     static const QString ICON_CONNECTED;
@@ -129,22 +127,4 @@ private:
     static const QString SUFFIX_WLAN;
 };
 
-class KNemoCheckListItem : public QObject, public QCheckListItem
-{
-    Q_OBJECT
-public:
-    KNemoCheckListItem( QListView* view )
-        : QCheckListItem( view, QString::null, QCheckListItem::CheckBox )
-    {}
-
-signals:
-    void stateChanged( KNemoCheckListItem*, bool );
-
-protected:
-    void stateChange( bool state )
-    {
-        emit stateChanged( this, state );
-    }
-};
-
 #endif // CONFIGDIALOG_H
