@@ -1,5 +1,6 @@
 /* This file is part of KNemo
    Copyright (C) 2006 Percy Leonhardt <percy@eris23.de>
+   Copyright (C) 2009 John Stamp <jstamp@users.sourceforge.net>
 
    KNemo is free software; you can redistribute it and/or modify
    it under the terms of the GNU Library General Public License as
@@ -17,32 +18,28 @@
    Boston, MA 02110-1301, USA.
 */
 
-#include <stdio.h>
-#include <iwlib.h>
-//#include <net/if.h>
-#include <string.h>
 #include <arpa/inet.h>
 #include <sys/ioctl.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
+#include <unistd.h>
 
-#include <qmap.h>
-#include <qdir.h>
-#include <qregexp.h>
-#include <qstringlist.h>
+#include <QDir>
 
-#include <kdebug.h>
-#include <kprocess.h>
 #include <kio/global.h>
 
+#include "config.h"
 #include "sysbackend.h"
 
-#include "config.h"
+#ifdef HAVE_LIBIW
+#include <iwlib.h>
+#else
+#include <net/if.h>
+#endif
 
+#define RTF_GATEWAY 0x0002
 #define SYSPATH "/sys/class/net/"
 #define PROCROUTE "/proc/net/route"
 
-SysBackend::SysBackend( QDict<Interface>& interfaces )
+SysBackend::SysBackend( QHash<QString, Interface *>& interfaces )
     : BackendBase( interfaces )
 {
 }
@@ -51,7 +48,7 @@ SysBackend::~SysBackend()
 {
 }
 
-BackendBase* SysBackend::createInstance( QDict<Interface>& interfaces )
+BackendBase* SysBackend::createInstance( QHash<QString, Interface *>& interfaces )
 {
     return new SysBackend( interfaces );
 }
@@ -61,13 +58,10 @@ void SysBackend::update()
     QDir dir( SYSPATH );
     QStringList ifList = dir.entryList( QDir::Dirs );
 
-    QDictIterator<Interface> ifIt( mInterfaces );
-    for ( ; ifIt.current(); ++ifIt )
+    foreach ( QString key, mInterfaces.keys() )
     {
-        QString key = ifIt.currentKey();
-        Interface* interface = ifIt.current();
-
-        if ( ifList.find( key ) == ifList.end() )
+        Interface *interface = mInterfaces.value( key );
+        if ( !ifList.contains( key ) )
         {
             // The interface does not exist. Meaning the driver
             // isn't loaded and/or the interface has not been created.
@@ -120,7 +114,7 @@ void SysBackend::update()
 
 bool SysBackend::readNumberFromFile( const QString& fileName, unsigned int& value )
 {
-    FILE* file = fopen( fileName.latin1(), "r" );
+    FILE* file = fopen( fileName.toLatin1(), "r" );
     if ( file != NULL )
     {
         if ( fscanf( file, "%ul", &value ) > 0 )
@@ -137,7 +131,7 @@ bool SysBackend::readNumberFromFile( const QString& fileName, unsigned int& valu
 bool SysBackend::readStringFromFile( const QString& fileName, QString& string )
 {
     char buffer[64];
-    FILE* file = fopen( fileName.latin1(), "r" );
+    FILE* file = fopen( fileName.toLatin1(), "r" );
     if ( file != NULL )
     {
         if ( fscanf( file, "%s", buffer ) > 0 )
@@ -243,21 +237,30 @@ void SysBackend::updateInterfaceData( const QString& ifName, InterfaceData& data
         {
             data.hwAddress = hwAddress;
         }
-    }
 
-    // for the default gateway we use the proc filesystem
-    QFile routeFile( PROCROUTE );
-    if ( routeFile.open( IO_ReadOnly ) )
-    {
-        QString routeData( routeFile.readAll().data() );
-        QRegExp regExp( ".*\\s+[\\w\\d]{8}\\s+([\\w\\d]{8})\\s+0+3" );
-        if ( regExp.search( routeData ) > -1 )
+        // for the default gateway we use the proc filesystem
+        QFile routeFile( PROCROUTE );
+        if ( routeFile.open( QIODevice::ReadOnly ) )
         {
-            bool ok;
-            struct in_addr in;
-            in.s_addr = regExp.cap( 1 ).toULong( &ok, 16 );
-            data.defaultGateway = inet_ntoa( in );
+            QString routeData( routeFile.readAll().data() );
+            QStringList routeEntries = routeData.split( "\n" );
+            QStringList::Iterator it;
+            for ( it = routeEntries.begin(); it != routeEntries.end(); ++it )
+            {
+                QRegExp regExp( ".*\\s+[\\w\\d]{8}\\s+([\\w\\d]{8})\\s+(\\d{4})" );
+                if (   ( regExp.indexIn( *it ) > -1 )
+                    && ( regExp.cap( 2 ).toUInt() & RTF_GATEWAY ) )
+                {
+                    bool ok;
+                    struct in_addr in;
+                    in.s_addr = regExp.cap( 1 ).toULong( &ok, 16 );
+                    data.defaultGateway = inet_ntoa( in );
+                    break;
+                }
+            }
+            routeFile.close();
         }
+
     }
 
     // use ioctls for the rest
@@ -265,7 +268,7 @@ void SysBackend::updateInterfaceData( const QString& ifName, InterfaceData& data
     struct ifreq ifr;
     if ( ( fd = socket(AF_INET, SOCK_DGRAM, 0) ) > -1 )
     {
-        strcpy( ifr.ifr_name, ifName.latin1() );
+        strcpy( ifr.ifr_name, ifName.toLatin1() );
         ifr.ifr_addr.sa_family = AF_INET;
         if ( ioctl( fd, SIOCGIFADDR, &ifr ) > -1 )
         {
@@ -285,6 +288,7 @@ void SysBackend::updateInterfaceData( const QString& ifName, InterfaceData& data
         {
             data.subnetMask = inet_ntoa(((struct sockaddr_in*)&ifr.ifr_netmask)->sin_addr);
         }
+        close( fd );
     }
 }
 
@@ -298,18 +302,19 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
         data.linkQuality = QString::number( link );
     }
 
+#ifdef HAVE_LIBIW
     // The following code was taken from iwconfig.c and iwlib.c.
     int fd;
     if ( ( fd = iw_sockets_open() ) > 0 )
     {
         struct iwreq wrq;
         char buffer[128];
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWFREQ, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWFREQ, &wrq ) >= 0 )
         {
             int channel = -1;
             double freq = iw_freq2float( &( wrq.u.freq ) );
             struct iw_range range;
-            if( iw_get_range_info( fd, ifName.latin1(), &range ) >= 0 )
+            if( iw_get_range_info( fd, ifName.toLatin1(), &range ) >= 0 )
             {
                 if ( freq < KILO )
                 {
@@ -330,7 +335,7 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
         wrq.u.essid.pointer = (caddr_t) essid;
         wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
         wrq.u.essid.flags = 0;
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWESSID, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWESSID, &wrq ) >= 0 )
         {
             if ( wrq.u.data.flags > 0 )
             {
@@ -342,7 +347,7 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
             }
         }
 
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWAP, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWAP, &wrq ) >= 0 )
         {
             char ap_addr[128];
             iw_ether_ntop( (const ether_addr*) wrq.u.ap_addr.sa_data, ap_addr);
@@ -353,7 +358,7 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
         wrq.u.essid.pointer = (caddr_t) essid;
         wrq.u.essid.length = IW_ESSID_MAX_SIZE + 1;
         wrq.u.essid.flags = 0;
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWNICKN, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWNICKN, &wrq ) >= 0 )
         {
             if ( wrq.u.data.length > 1 )
             {
@@ -365,7 +370,7 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
             }
         }
 
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWRATE, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWRATE, &wrq ) >= 0 )
         {
             iwparam bitrate;
             memcpy (&(bitrate), &(wrq.u.bitrate), sizeof (iwparam));
@@ -373,7 +378,7 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
             data.bitRate = buffer;
         }
 
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWMODE, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWMODE, &wrq ) >= 0 )
         {
             int mode = wrq.u.mode;
             if ( mode < IW_NUM_OPER_MODE && mode >= 0 )
@@ -390,7 +395,7 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
         wrq.u.data.pointer = (caddr_t) key;
         wrq.u.data.length = IW_ENCODING_TOKEN_MAX;
         wrq.u.data.flags = 0;
-        if ( iw_get_ext( fd, ifName.latin1(), SIOCGIWENCODE, &wrq ) >= 0 )
+        if ( iw_get_ext( fd, ifName.toLatin1(), SIOCGIWENCODE, &wrq ) >= 0 )
         {
             if ( ( wrq.u.data.flags & IW_ENCODE_DISABLED ) || ( wrq.u.data.length == 0 ) )
             {
@@ -407,5 +412,6 @@ void SysBackend::updateWirelessData( const QString& ifName, WirelessData& data )
         }
         close( fd );
     }
+#endif
 }
 
