@@ -1,5 +1,6 @@
 /* This file is part of KNemo
    Copyright (C) 2004, 2006 Percy Leonhardt <percy@eris23.de>
+   Copyright (C) 2009 John Stamp <jstamp@users.sourceforge.net>
 
    KNemo is free software; you can redistribute it and/or modify
    it under the terms of the GNU Library General Public License as
@@ -20,9 +21,7 @@
 #ifndef INTERFACE_H
 #define INTERFACE_H
 
-#include <qobject.h>
-#include <qstring.h>
-#include <qdatetime.h>
+#include <QDate>
 
 #include "data.h"
 #include "global.h"
@@ -30,7 +29,7 @@
 #include "interfacemonitor.h"
 
 class QTimer;
-class SignalPlotter;
+class InterfacePlotterDialog;
 class InterfaceStatistics;
 class InterfaceStatusDialog;
 class InterfaceStatisticsDialog;
@@ -172,6 +171,11 @@ public slots:
      */
     void showStatisticsDialog();
 
+    /*
+     * Reset data when PPP interface is disconnected
+     */
+    void resetData( int state );
+
 private slots:
     /**
      * Start the uptimer when the interface is connected
@@ -184,9 +188,9 @@ private slots:
     void updatePlotter();
 
     /**
-     * Configure the signal plotter with user settings
+     * Update the tooltip and status dialog
      */
-    void configurePlotter();
+    void updateDetails();
 
 private:
     /**
@@ -199,18 +203,15 @@ private:
      */
     void stopStatistics();
 
-    enum VisibleBeams
-    {
-        NONE = 0,
-        INCOMING_TRAFFIC = 1,
-        OUTGOING_TRAFFIC = 2,
-        BOTH = 3
-    };
+    /**
+     * The following function is taken from ksystemtray.cpp for
+     * correct show, raise, focus and hide of status dialog and
+     * signal plotter.
+     */
+    void activateOrHide( QWidget* widget, bool onlyActivate = false );
 
     int mType;
     int mState;
-    int mOutgoingPos;
-    int mIncomingPos;
     QString mName;
     QTimer* mPlotterTimer;
     QDateTime mStartTime;
@@ -222,8 +223,7 @@ private:
     WirelessData mWirelessData;
     InterfaceStatusDialog* mStatusDialog;
     InterfaceStatisticsDialog* mStatisticsDialog;
-    SignalPlotter* mPlotter;
-    VisibleBeams mVisibleBeams;
+    InterfacePlotterDialog* mPlotterDialog;
     const GeneralData& mGeneralData;
     const PlotterSettings& mPlotterSettings;
 };
